{% raw %}
    <!DOCTYPE html>
    <html lang="en">
        <head>
            <meta charset="UTF-8">
            <meta name="viewport" content="width=device-width, initial-scale=1.0">
            <meta http-equiv="X-UA-Compatible" content="ie=edge">
            <title>{% block title %}Document{% endblock %}</title>
        </head>
        <body>
            {% block content %}
            {% endblock %}
        </body>
    </html>
{% endraw %}
