{% raw %}
    {% extends 'base.html' %}
    {%block title %}Congratulations - Render Engine{% endblock %}
    {% block content %}
    <h1>{{SITE_TITLE}}</h1>
    {% if SITE_DESCRIPTION %}
        <p>{{SITE_DESCRIPTION}}</p>
    {% endif %}

    <h2>Congratulations{% if SITE_AUTHOR %} {{SITE_AUTHOR}}{% endif %}!</h2>
    <p>You've Built your Site with Render_Engine</p>

    {% if collection %}
    <p>You have also created a collection object</p>
        <ul>

        </ul>
    {% endif %}
    {% endblock %}
{% endraw %}
