from .blog import Blog
from .collection import Collection
from .page import Page

# TODO: #70 site is a reserved keyword in python.
# It should possibly become a new module name
from .site import Site 
